/* extension.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/* exported init */

const Gettext = imports.gettext;
const { GObject } = imports.gi;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;

const Domain = Gettext.domain(Me.metadata.uuid);
const _ = Domain.gettext;

const DisplayBackend = Me.imports.dbus;

// Round scale values to multiple of 25 % just like Gnome Settings
function scaleToPercentage(scale) {
    return Math.floor(scale * 4 + 0.5) * 25;
}

const DisplayScaleSubMenu = GObject.registerClass(
    class DisplayScaleSubMenu extends PopupMenu.PopupSubMenuMenuItem {

        _init() {
            // Set submenu name and icon
            super._init(_('Display Scale'), true);
            this.icon.set_icon_name('video-display-symbolic');

            this._displayScaleSwitcher = new DisplayBackend.DisplayScaleSwitcher();

            this.menu.connect('open-state-changed', (menu, isOpen) => {
                if (isOpen) {
                    this._updateDisplayList();
                }
            });

            this._addDummyItem(_('Obtaining display info...'));
        }

        _addDummyItem(message) {
            const item = new PopupMenu.PopupMenuItem(message);
            item.label.get_clutter_text().set_line_wrap(true);
            this.menu.addMenuItem(item);
        }

        _updateDisplayList() {
            this.menu.removeAll();

            const displays = this._displayScaleSwitcher.getDisplayInfo();

            if (displays === null) {
                this._addDummyItem(_('Unable to get display info.'));
                return;
            }

            // Current version only supports single display.
            if (displays.length > 1) {
                this._addDummyItem(_('Multiple displays detected. This is not supported by the extension.'));
                return;
            }
            else {
                const display = displays[0];

                for (let scale of display.scales) {
                    const scaleItem = new PopupMenu.PopupMenuItem(scaleToPercentage(scale) + ' %');
                    
                    scaleItem.connect('activate', () => {
                        this._displayScaleSwitcher.setDisplayScale(scale);
                    });

                    scaleItem.setOrnament(scale === display.currentScale
                        ? PopupMenu.Ornament.DOT
                        : PopupMenu.Ornament.NONE);
        
                    this.menu.addMenuItem(scaleItem);
                }
            }
        }
    });

class Extension {
    constructor(uuid) {
        this._uuid = uuid;
        this._menu = null;

        ExtensionUtils.initTranslations(uuid);
    }

    enable() {
        this._menu = new DisplayScaleSubMenu();
        
        const menuItems = Main.panel.statusArea.aggregateMenu.menu._getMenuItems();

        let separatorCounter = 0;
        for (let i = 0; i < menuItems.length; i++) {
            if(menuItems[i] instanceof PopupMenu.PopupSeparatorMenuItem) {
                separatorCounter++;
                if (separatorCounter == 2) {
                    Main.panel.statusArea.aggregateMenu.menu.addMenuItem(this._menu, i);
                }
            }
        }
    }

    disable() {
        this._menu.destroy();
        this._menu = null;
    }
}

function init(meta) {
    return new Extension(meta.uuid);
}
